/**************************************************
 * How to compile:
 *	arm-linux-gnueabihf-gcc SoCTest03.c -o SoCTest03 -lm -I/opt/altera/15.0/embedded/ip/altera/hps/altera_hps/hwlib/include/ -I/opt/altera/15.0/embedded/ip/altera/hps/altera_hps/hwlib/include/soc_cv_av/
 **************************************************/

#ifndef soc_cv_av
#define	soc_cv_av
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/mman.h>
#include "hwlib.h"
#include "socal/socal.h"
#include "socal/hps.h"
#include "socal/alt_gpio.h"
#include "hps_0.h"

#define HW_REGS_SPAN ( 0x04000000 )
#define HW_REGS_MASK ( HW_REGS_SPAN - 1 )

#define BASE_NUMBER	0xFFFFFFFF		// base numerica intera
#define TS			100e-9			// tempo di sample a 10MHz

void printHelp();

volatile unsigned long *h2p_lw_led_addr=NULL;
volatile unsigned long *LPf_Ku = NULL;			// indirizzo del parametro Ku
volatile unsigned long *LPf_Ky = NULL;			// indirizzo del parametro Ky
volatile signed long *LPf_u = NULL;				// indirizzo dell'ingresso u
volatile signed long *LPf_y = NULL;				// indirizzo dell'uscita y


int main(int argc, char **argv)
{
	void *virtual_base;
	int fd;
	unsigned led;
	char ch;
	char str[20];
	double tau;
	double U;
	int32_t		u, y;
	uint32_t	Ku, Ky;
	
	printf("ALT_STM_OFST = 0x%8X\nALT_LWFPGASLVS_OFST = 0x%8X\nLED_BASE = 0x%0X\n(ALT_LWFPGASLVS_OFST + LED_BASE) & HW_REGS_MASK = 0x%08X\n\n",
		   ALT_STM_OFST, ALT_LWFPGASLVS_OFST, LED_BASE, (ALT_LWFPGASLVS_OFST + LED_BASE) & (HW_REGS_MASK));
	
	// Apertura del device che permette l'accesso diretto alla memoria
	if((fd = open("/dev/mem", (O_RDWR | O_SYNC))) == -1) {
		printf("ERROR: could not open \"/dev/mem\"...\n");
		return(1);
	}
	
	// Mappatura della memria a partire da ALT_STM_OFST (System Trace Macrocell (STM) Module Address Map)
	// Questo indirizzo è esattamente qeullo di inizio dell'area destinata alle periferiche: 0xFC00_0000
	virtual_base = mmap( NULL, HW_REGS_SPAN, (PROT_READ | PROT_WRITE), MAP_SHARED, fd, ALT_STM_OFST);	
	if(virtual_base == MAP_FAILED) {
		printf("ERROR: mmap() failed...\n");
		close(fd);
		return(1);
	}
	
	// Costruzione degli indirizzi
	h2p_lw_led_addr = virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST+LED_BASE);
	LPf_Ku = virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST + FILT_KU_BASE);
	LPf_Ky = virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST + FILT_KY_BASE);
	LPf_u =  virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST + FILT_U_BASE);
	LPf_y =  virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST + FILT_Y_BASE);
	
	ch = 0;
	while(ch != 'Q' && ch != 'q' && ch != 'E' && ch != 'e' && ch != 27)
	{
		switch(ch)
		{
			case 'l':
			case 'L':
				printf("LED value = ");
				scanf("%20s", str);
				led = atoi(str);
				printf("\tWrite LED = %i\n", led);
				*h2p_lw_led_addr = led;
				break;
				
			case 'f':
			case 'F':
				printf("Filter Time constant = ");
				scanf("%20s", str);
				tau = atof(str);
				if(tau > 0)
				{
					Ku = 1+(double)(0.5 - 0.5*exp(-TS / tau)) * BASE_NUMBER;
					Ky = (double)exp(-TS / tau) * BASE_NUMBER;
				}
				else
				{
					// Disabilitazione del filtro
					Ku = 1 + BASE_NUMBER >> 1;
					Ky = 0;
				}
				printf("\tWrite\tKu = %8X\n\t\tKy = %8X\n", Ku, Ky);
				led = ~*h2p_lw_led_addr;	// leggo il valore negato di led
				*LPf_Ku = Ku;
				*LPf_Ky = Ky;
				*h2p_lw_led_addr = led;		// scrivo led creando il trigger per signal tap
				break;
			
			case 'u':
			case 'U':
				printf("Input Filter (float per-unit) = ");
				scanf("%20s", str);
				U = atof(str);
				if(U > +1)
					U = +1;
				else
				if(U < -1)
					U = -1;
				printf("\tWrite U = %.3f [%i]\n", U, (float)U*(BASE_NUMBER/2));
				led = ~*h2p_lw_led_addr;	// leggo il valore negato di led
				*LPf_u = (double)U*(BASE_NUMBER/2);
				*h2p_lw_led_addr = led;		// scrivo led creando il trigger per signal tap
				break;
				
			case 'y':
			case 'Y':
				printf("Ku Filt = %f\nKy Filt = %f\nOutput filter: %f\n",
					   (float)*LPf_Ku/(float)BASE_NUMBER,
					   (float)*LPf_Ky/(float)BASE_NUMBER,
					   (float)*LPf_y/(float)BASE_NUMBER*0.5);
				break;

			default:
				printHelp();
		}

		printf("===============================\n\n");
		while((ch = getchar()) == '\n');
	}
	
	if(munmap(virtual_base, HW_REGS_SPAN ) != 0) {
		printf("ERROR: munmap() failed...\n");
		close(fd);
		return 1;
	}
	
	close(fd);
	return 0;
}

void printHelp()
{
	printf(
	"\nInsert command:\n"
	"-------------------------------\n"
	"L\tChange LED value\n"
	"F\tConfig Filter coefficients\n"
	"U\tSet Input Filter (per-unit)\n"
	"Y\tRead Y value\n"
	"E/Q\nExit\n"
	"-------------------------------\n"
	);
}
